/**
 * vue-cesium - https://github.com/zouyaoji/vue-cesium
 * Vue 2.x components for CesiumJS.
 *
 * @package VueCesium
 * @author zouyaoji <370681295@qq.com>
 * @version 2.4.2
 * @license MIT
 * @homepage https://zouyaoji.top/vue-cesium
 * @copyright (c) 2018-2023, zouyaoji <370681295@qq.com>
 */
import _asyncToGenerator from '@babel/runtime-corejs2/helpers/esm/asyncToGenerator';
import _regeneratorRuntime from '@babel/runtime-corejs2/regenerator';
import cmp from '../../mixins/virtualCmp';
import { polygonHierarchy } from '../../mixins/mixinProps';
import { makeColor } from '../../utils/cesiumHelpers';
import { getVmListenerName } from '../../utils/util';
import * as ClassificationPrimitive from '../../primitive/classification';
import * as GeometryInstance from '../../geometryInstance';
import * as PolygonGeometry from '../../geometryInstance/polygon';

var script = {
  name: 'vc-analytics-flood',
  data: function data() {
    return {
      attributes: undefined,
      extrudedHeight: -1
    };
  },
  mixins: [cmp, polygonHierarchy],
  props: {
    minHeight: {
      type: Number,
      default: -1
    },
    maxHeight: {
      type: Number,
      default: 8888
    },
    loop: {
      type: Boolean,
      default: false
    },
    speed: {
      type: Number,
      default: 10
    },
    color: {
      type: [Object, Array, String],
      default: 'rgba(40,150,200,0.6)'
    }
  },
  watch: {
    minHeight: function minHeight(val) {
      this.extrudedHeight = val;
    }
  },
  methods: {
    createCesiumObject: function createCesiumObject() {
      var _this = this;

      return _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee() {
        var Cesium, color, ColorGeometryInstanceAttribute;
        return _regeneratorRuntime.wrap(function _callee$(_context) {
          while (1) {
            switch (_context.prev = _context.next) {
              case 0:
                Cesium = _this.Cesium, color = _this.color;
                ColorGeometryInstanceAttribute = Cesium.ColorGeometryInstanceAttribute;
                _this.attributes = {
                  color: ColorGeometryInstanceAttribute.fromColor(makeColor(color))
                };
                return _context.abrupt("return", true);

              case 4:
              case "end":
                return _context.stop();
            }
          }
        }, _callee);
      }))();
    },
    mount: function mount() {
      var _this2 = this;

      return _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee2() {
        return _regeneratorRuntime.wrap(function _callee2$(_context2) {
          while (1) {
            switch (_context2.prev = _context2.next) {
              case 0:
                _this2.viewer.clock.onTick.addEventListener(_this2.onClockTick);

                return _context2.abrupt("return", true);

              case 2:
              case "end":
                return _context2.stop();
            }
          }
        }, _callee2);
      }))();
    },
    unmount: function unmount() {
      var _this3 = this;

      return _asyncToGenerator( /*#__PURE__*/_regeneratorRuntime.mark(function _callee3() {
        return _regeneratorRuntime.wrap(function _callee3$(_context3) {
          while (1) {
            switch (_context3.prev = _context3.next) {
              case 0:
                _this3.viewer.clock.onTick.removeEventListener(_this3.onClockTick);

                _this3.extrudedHeight = -1;
                _this3.flooding = false;
                return _context3.abrupt("return", true);

              case 4:
              case "end":
                return _context3.stop();
            }
          }
        }, _callee3);
      }))();
    },
    onClockTick: function onClockTick(e) {
      if (this.flooding) {
        if (this.extrudedHeight <= this.maxHeight) {
          this.extrudedHeight += this.speed;
          this.stoped = false;
        } else {
          var listener = getVmListenerName.call(this, 'stop');
          listener && this.$emit('stop', this.$refs.primitive);
          this.stoped = true;

          if (this.loop) {
            this.extrudedHeight = this.minHeight;
          } else {
            this.flooding = false;
          }
        }
      }
    },
    start: function start(height) {
      this.extrudedHeight = Cesium.defined(height) ? height : this.minHeight;
      this.flooding = true;
    },
    pause: function pause() {
      this.flooding = !this.flooding;

      if (this.stoped) {
        this.extrudedHeight = this.minHeight;
      }
    },
    stop: function stop() {
      this.extrudedHeight = -1;
      this.flooding = false;
    }
  }
};

/* script */
var __vue_script__ = script;
/* template */

var __vue_render__ = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('vc-primitive-classification', {
    ref: "primitive",
    attrs: {
      "asynchronous": false
    }
  }, [_c('vc-instance-geometry', {
    attrs: {
      "attributes": _vm.attributes
    }
  }, [_c('vc-geometry-polygon', {
    attrs: {
      "extrudedHeight": _vm.extrudedHeight,
      "polygonHierarchy": _vm.polygonHierarchy
    }
  })], 1)], 1);
};

var __vue_staticRenderFns__ = [];
/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = undefined;
/* functional template */

var __vue_is_functional_template__ = false;
/* component normalizer */

function __vue_normalize__(template, style, script, scope, functional, moduleIdentifier, createInjector, createInjectorSSR) {
  var component = (typeof script === 'function' ? script.options : script) || {}; // For security concerns, we use only base name in production mode.

  component.__file = "VcFlood.vue";

  if (!component.render) {
    component.render = template.render;
    component.staticRenderFns = template.staticRenderFns;
    component._compiled = true;
    if (functional) component.functional = true;
  }

  component._scopeId = scope;

  return component;
}
/* style inject */

/* style inject SSR */


var VcFlood = __vue_normalize__({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__);

function plugin(Vue) {

  if (plugin.installed) {
    return;
  }

  plugin.installed = true;
  Vue.use(ClassificationPrimitive);
  Vue.use(GeometryInstance);
  Vue.use(PolygonGeometry);
  Vue.component(VcFlood.name, VcFlood);
}

export default plugin;
export { VcFlood, plugin as install };
